/*                        sst_clim.c
 *
 *   program to read the long-term SST monthly means and climatology
 *   derived by Shea, Trenberth, and Reynolds - 1990
 *   2x2 degree grid with one row and column overlap
 *   the array/geographic layout is:
 *   (1,1)=(0,-90);   (2,1)=(2,-90);   (3,1)=(4,-90); ....
 *   (179,1)=(356,-90); (180,1)=(358,-90);  (181,1)=(360,-90)[cyclic pt]
 *   (1,2)=(0,-88);   (2,2)=(2,-88);   (3,2)=(4,-88); ....
 *   (179,2)=(356,-88); (180,2)=(358,-88), etc
 *
 *   --> grid points at ice locations are set to -1.80
 *   --> grid points at land locations have erroneous sst values
 */

#include <stdio.h>
#include <limits.h>
#include <netcdf.h>

#define MLON 181
#define NLAT  91
#define GLMLON 360
#define GLMLAT 181
#define LANDVAL FILL_FLOAT
#define MISSVAL -FILL_FLOAT


void main(int argc, char *argv[])
{
  FILE *GLM, *SST;
  int   j, i, cdfid, sstid, latid, lonid, mon, nmo;
  long  sst_count[4] = {1, 1, GLMLAT, GLMLON},
        sst_start[4] = {0, 0, 0, 0},
        start = 0, dummy,
        imax =GLMLON, jmax = GLMLAT,
        landmask[GLMLON];
  float fl_range[2], sst[NLAT][MLON], hr_sst[GLMLAT][GLMLON];
  float lat[GLMLAT], lon[GLMLON];
  float fmin_max[2], fmin, fmax;

  if (argc > 1) {
    SST = stdin;
  } else if ( (SST = fopen("shea_sst.dat","r")) == NULL) {
    fprintf(stderr,"unable to open shea_sst.dat\n");
    exit (1);
  }

/* set the land grid pts to special value, input data 0 - water, 1 - land */
  if ( (GLM = fopen("/usr/local/gasp/data/glm.dat","r")) == NULL) {
    fprintf(stderr,"unable to open /usr/local/gasp/data/glm.dat\n");
    exit(1);
  }
  for (j=0; j<GLMLAT; ++j) {

    for (i=0; i<GLMLON/2; ++i)
      landmask[i] = (long) (getc(GLM) - '0');
    getc(GLM);
    for (i=GLMLON/2; i<GLMLON; ++i)
      landmask[i] = (long) (getc(GLM) - '0');
    getc(GLM);
    for (i=0; i<GLMLON; ++i)
      if (landmask[i])
	hr_sst[j][i] = LANDVAL;
      else
	hr_sst[j][i] = 0.0;
  }
  fclose(GLM);

  ncopts = NC_VERBOSE;
  if ((cdfid = ncopen("sst_clim.nc",NC_WRITE)) < 0) {
    fprintf(stderr,"unable to open sst_clim.nc/n");
    exit(1);
  }

  ncsetfill(cdfid,NC_NOFILL);

  fmin =  2e15 -1;
  fmax = -fmin;

  for (i=0;i<GLMLON; ++i) { 
    lon[i] = (float) i;
    if (lon[i] < fmin) fmin = lon[i];
    if (lon[i] > fmax) fmax = lon[i];
  }

  fmin_max[0] = fmin;
  fmin_max[1] = fmax;

  ncattput(cdfid, ncvarid(cdfid, "lon"), "min_max", NC_FLOAT, 2, fmin_max);
  ncvarput(cdfid, ncvarid(cdfid, "lon"), &start, &imax, (void *)lon);

  fmin =  2e15 -1;
  fmax = -fmin;

  for (j=0;j<=GLMLAT-1; ++j) { 
    lat[j] = -90.0 + (float) j;
    if (lat[j] < fmin) fmin = lat[j];
    if (lat[j] > fmax) fmax = lat[j];
  }

  fmin_max[0] = fmin;
  fmin_max[1] = fmax;

  ncattput(cdfid, ncvarid(cdfid, "lat"), "min_max", NC_FLOAT, 2, fmin_max);
  ncvarput(cdfid, ncvarid(cdfid, "lat"), &start, &jmax, (void *)lat);

  sst_start[1] = 0;
  sst_start[2] = 0;
  sst_start[3] = 0;
  fl_range[0] = FILL_FLOAT;
  fl_range[1] = -FILL_FLOAT;

  sstid = ncvarid(cdfid, "sst");

  for (mon=0;mon<12; ++mon) {
    fscanf(SST,"%d",&nmo);
    fprintf(stdout,"processing month - %2d, ",nmo);
    for (j=0; j<NLAT; ++j) {
      for (i=0; i<MLON; ++i) {
	fscanf(SST,"%f",&sst[j][i]);
	if (sst[j][i] < fl_range[0]) fl_range[0] = sst[j][i];
	if (sst[j][i] > fl_range[1]) fl_range[1] = sst[j][i];
      }
    }
    fprintf(stdout,"min/max sst %6.2f %6.2f \n",fl_range[0],fl_range[1]);
 
    for (j=0; j<GLMLAT; ++j)
      for (i=0; i<GLMLON; ++i)
	if (hr_sst[j][i] < LANDVAL - 1.0) {
	  if (!(i%2)) {
	    if (!(j%2))
	      hr_sst[j][i] = sst[j/2][i/2];
	    else
	      hr_sst[j][i] =
		(sst[(j-1)/2][i/2] + sst[(j+1)/2][i/2]) / 2.0;
	  } else {
	    if (!(j%2))
	      hr_sst[j][i] =
		(sst[j/2][(i-1)/2] + sst[j/2][(i+1)/2]) / 2.0;
	    else
	      hr_sst[j][i] =
		(sst[(j-1)/2][(i-1)/2] + sst[(j-1)/2][(i+1)/2] +
		 sst[(j+1)/2][(i-1)/2] + sst[(j+1)/2][(i+1)/2]) / 4.0;
	  }
	}
    sst_start[0] = mon;
    ncvarput(cdfid,sstid,sst_start,sst_count,(void *)hr_sst);
  }

  ncattput(cdfid, sstid, "min_max",NC_FLOAT, 2, fl_range);

  ncclose(cdfid);
  fprintf(stdout,"end of processing\n");
}
